//
//  MinimiseSim.swift
//
//  Listens for new Vensim windows whose title starts with "sim "
//  and minimises them instantly.  CPU use ~0% thanks to AXObserver.
//

import Cocoa
import ApplicationServices   // AX APIs

// ------- CONFIGURATION -------
let targetBundleID = "com.vensim.VensimDSSMC"
let prefix         = "sim "               // leading space intentional
// -----------------------------

// Keep the observer alive globally
var axObserver: AXObserver?

// Minimise window if its title matches our prefix
func maybeMinimise(_ element: AXUIElement) {
    var titleRef: CFTypeRef?
    if AXUIElementCopyAttributeValue(element,
                                     kAXTitleAttribute as CFString,
                                     &titleRef) == .success,
       let title = titleRef as? String,
       title.hasPrefix(prefix) {

        AXUIElementSetAttributeValue(element,
                                     kAXMinimizedAttribute as CFString,
                                     kCFBooleanTrue)
    }
}

// Called by AXObserver when any window is created
let callback: AXObserverCallback = { _, element, notification, _ in
    if notification as String == kAXWindowCreatedNotification as String {
        maybeMinimise(element)
    }
}

// Attach an AXObserver to the target PID
func attachObserver(to pid: pid_t) {
    let appElement = AXUIElementCreateApplication(pid)
    var observer: AXObserver?
    if AXObserverCreate(pid, callback, &observer) == .success,
       let obs = observer {

        axObserver = obs                                     // keep alive
        AXObserverAddNotification(obs, appElement,
                                  kAXWindowCreatedNotification as CFString,
                                  nil)
        // Catch any "sim " windows that existed *before* we attached
        var windowsRef: CFTypeRef?
        if AXUIElementCopyAttributeValue(appElement,
                                         kAXWindowsAttribute as CFString,
                                         &windowsRef) == .success,
           let windows = windowsRef as? [AXUIElement] {
            windows.forEach { maybeMinimise($0) }
        }

        // Start listening
        CFRunLoopAddSource(CFRunLoopGetCurrent(),
                           AXObserverGetRunLoopSource(obs),
                           .defaultMode)
        print("✓ Observer attached to Vensim (pid \(pid))")
    } else {
        fputs("⚠︎ Failed to attach AXObserver\n", stderr)
        exit(1)
    }
}

// If Vensim is already running, attach immediately
if let app = NSRunningApplication
              .runningApplications(withBundleIdentifier: targetBundleID)
              .first {
    attachObserver(to: app.processIdentifier)
}

// Also watch for future launches of Vensim
NSWorkspace.shared.notificationCenter.addObserver(
    forName: NSWorkspace.didLaunchApplicationNotification,
    object: nil,
    queue: nil) { note in
        guard let launched =
                note.userInfo?[NSWorkspace.applicationUserInfoKey]
                  as? NSRunningApplication,
              launched.bundleIdentifier == targetBundleID else { return }
        attachObserver(to: launched.processIdentifier)
    }

// Run forever
CFRunLoopRun()
